@@ -51,22 +51,42 @@
 #include "fileInt.h"
 #include "util.h"
 #include "str.h"
-#include "err.h"
 #include "vm_version.h"
 #include "localconfig.h"
 #include "hostinfo.h"
 #include "su.h"
-#include "hostType.h"
 
 #include "unicodeOperations.h"
 
+#if defined(VMX86_SERVER)
+#include "hostType.h"
+/*
+ * UserWorlds have to handle file locking slightly differently than COS
+ * applications, as they are in a different pid space.  The problem is that
+ * if a UserWorld tries to write its UserWorld pid to a lock file, a COS
+ * program will think the lock file is stale, as that pid won't be valid on
+ * the COS. Luckily, every UW is attached to a COS proxy, so, for the
+ * purposes of this file, the UW can use its COS proxy's pid as its pid when
+ * locking files. This way, if a COS app looks up the proxy pid, it will be
+ * valid and the COS app will wait for the lock.
+ */
+#include "uwvmk.h"
+#endif
+
 #define LOGLEVEL_MODULE main
 #include "loglevel_user.h"
 
 #define DEVICE_LOCK_DIR "/var/lock"
 
-#define LOG_MAX_PROC_NAME  64
+/*
+ * Parameters used by the file library.
+ */
+typedef struct FileLockOptions {
+   int lockerPid;
+   Bool userWorld;
+} FileLockOptions;
 
+static FileLockOptions fileLockOptions = { 0, FALSE };
 
 /*
  * XXX
@@ -77,6 +97,30 @@
  * into the log file.
  */
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * FileLock_Init --
+ *
+ *      Initialize the file locking library.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Allows the user to enable locking on random file systems.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+FileLock_Init(int lockerPid,            // IN
+              Bool userWorld)           // IN
+{
+   fileLockOptions.lockerPid = lockerPid;
+   fileLockOptions.userWorld = userWorld;
+}
+
 #if !defined(__FreeBSD__) && !defined(sun)
 /*
  *----------------------------------------------------------------------
@@ -96,27 +140,24 @@
  */
 
 static Bool
-IsLinkingAvailable(const char *fileName)  // IN:
+IsLinkingAvailable(const char *fileName)     // IN:
 {
    struct statfs buf;
-   int status;
+   int           status;
 
    ASSERT(fileName);
 
-   /*
-    * Don't use linking on ESX/VMFS... the overheads are expensive and this
-    * path really isn't used.
-    */
-
-   if (HostType_OSIsVMK()) {
+#if defined(VMX86_SERVER)
+   // Never supported on VMvisor
+   if (HostType_OSIsPureVMK()) {
       return FALSE;
    }
+#endif
 
    status = statfs(fileName, &buf);
 
    if (status == -1) {
-      Log(LGPFX" Bad statfs using %s (%s).\n", fileName,
-          Err_Errno2String(errno));
+      Log(LGPFX" Bad statfs using %s (%s).\n", fileName, strerror(errno));
 
       return FALSE;
    }
@@ -153,6 +194,7 @@ IsLinkingAvailable(const char *fileName)  // IN:
    case TMPFS_SUPER_MAGIC:
    case JFS_SUPER_MAGIC:
         return TRUE;                        // these are known to work
+   case VMFS_SUPER_MAGIC:
    case SMB_SUPER_MAGIC:
    case MSDOS_SUPER_MAGIC:
         return FALSE;
@@ -170,6 +212,44 @@ IsLinkingAvailable(const char *fileName)  // IN:
 }
 
 
+/*
+ *---------------------------------------------------------------------------
+ *
+ * FileLockGetPid --
+ *
+ *      Returns the pid of the main thread if we're running in a
+ *      multithreaded process, otherwise return the result of getpid().
+ *
+ * Results:
+ *      a pid.
+ *
+ * Side effects:
+ *      None.
+ *
+ *---------------------------------------------------------------------------
+ */
+
+static int
+FileLockGetPid(void)
+{
+#if defined(VMX86_SERVER)
+   /*
+    * For a UserWorld, we want to get this cartel's proxy's cos pid.
+    */
+   if (fileLockOptions.userWorld) {
+      int pid = 0;
+
+      VMKernel_GetLockPid(&pid);
+
+      return pid;
+   }
+#endif
+
+   return fileLockOptions.lockerPid > 0 ?
+                         fileLockOptions.lockerPid : (int) getpid();
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -187,7 +267,7 @@ IsLinkingAvailable(const char *fileName)  // IN:
  */
 
 static Bool
-RemoveStaleLockFile(const char *lockFileName)  // IN:
+RemoveStaleLockFile(const char *lockFileName)      // IN:
 {
    uid_t uid;
    int ret;
@@ -206,7 +286,7 @@ RemoveStaleLockFile(const char *lockFileName)  // IN:
 
    if (ret < 0) {
       Warning(LGPFX" Failed to remove stale lock file %s (%s).\n",
-              lockFileName, Err_Errno2String(saveErrno));
+              lockFileName, strerror(saveErrno));
 
       return FALSE;
    }
@@ -232,9 +312,9 @@ RemoveStaleLockFile(const char *lockFileName)  // IN:
  */
 
 static int
-GetLockFileValues(const char *lockFileName,  // IN:
-                  int *pid,                  // OUT:
-                  char *hostID)              // OUT:
+GetLockFileValues(const char *lockFileName, // IN:
+                  int *pid,                 // OUT:
+                  char *hostID)             // OUT:
 {
    char *p;
    int  saveErrno;
@@ -255,7 +335,7 @@ GetLockFileValues(const char *lockFileName,  // IN:
 
    if (lockFile == NULL) {
       Warning(LGPFX" Failed to open existing lock file %s (%s).\n",
-              lockFileName, Err_Errno2String(saveErrno));
+              lockFileName, strerror(saveErrno));
 
       return (saveErrno == ENOENT) ? 0 : -1;
    }
@@ -267,7 +347,7 @@ GetLockFileValues(const char *lockFileName,  // IN:
 
    if (p == NULL) {
       Warning(LGPFX" Failed to read line from lock file %s (%s).\n",
-              lockFileName, Err_Errno2String(saveErrno));
+              lockFileName, strerror(saveErrno));
 
       deleteLockFile = TRUE;
    } else {
@@ -320,6 +400,12 @@ FileLockIsValidProcess(int pid)  // IN:
    int err;
    Bool ret;
 
+#if defined(VMX86_SERVER)
+   if (fileLockOptions.userWorld) {
+      return (VMKernel_IsLockPidAlive(pid) == 0) ? TRUE : FALSE;
+   }
+#endif
+
    uid = Id_BeginSuperUser();
    err = (kill(pid, 0) == -1) ? errno : 0;
    Id_EndSuperUser(uid);
@@ -382,7 +468,7 @@ FileLockCreateLockFile(const char *lockFileName,  // IN:
 
       if (lockFD == -1) {
          Log(LGPFX" Failed to create new lock file %s (%s).\n",
-             lockFileLink, Err_Errno2String(saveErrno));
+             lockFileLink, strerror(saveErrno));
 
          return (saveErrno == EEXIST) ? 0 : -1;
       }
@@ -403,7 +489,7 @@ FileLockCreateLockFile(const char *lockFileName,  // IN:
 
       if (lockFD == -1) {
          Log(LGPFX" Failed to create new lock file %s (%s).\n",
-             lockFileName, Err_Errno2String(saveErrno));
+             lockFileName, strerror(saveErrno));
 
          return (saveErrno == EEXIST) ? 0 : -1;
       }
@@ -416,7 +502,7 @@ FileLockCreateLockFile(const char *lockFileName,  // IN:
 
    if (err != strlen(uniqueID)) {
       Warning(LGPFX" Failed to write to new lock file %s (%s).\n",
-              lockFileName, Err_Errno2String(saveErrno));
+              lockFileName, strerror(saveErrno));
       status = -1;
       goto exit;
    }
@@ -437,7 +523,7 @@ exit:
 
       if (err < 0) {
          Warning(LGPFX" Failed to remove temporary lock file %s (%s).\n",
-                 lockFileLink, Err_Errno2String(errno));
+                 lockFileLink, strerror(errno));
       }
 
    }
@@ -469,7 +555,7 @@ exit:
  */
 
 int
-FileLock_LockDevice(const char *deviceName)  // IN:
+FileLock_LockDevice(const char *deviceName)   // IN:
 {
    const char *hostID;
    char       uniqueID[1000];
@@ -484,14 +570,14 @@ FileLock_LockDevice(const char *deviceName)  // IN:
                                    deviceName);
 
    lockFileLink = Str_SafeAsprintf(NULL, "%s/LTMP..%s.t%05d", DEVICE_LOCK_DIR,
-                                   deviceName, getpid());
+                                   deviceName, FileLockGetPid());
 
    LOG(1, ("Requesting lock %s (temp = %s).\n", lockFileName,
            lockFileLink));
 
    hostID = FileLockGetMachineID();
    Str_Sprintf(uniqueID, sizeof uniqueID, "%d %s\n",
-               getpid(), hostID);
+               FileLockGetPid(), hostID);
 
    while ((status = FileLockCreateLockFile(lockFileName, lockFileLink,
                                            uniqueID)) == 0) {
@@ -579,7 +665,7 @@ FileLock_UnlockDevice(const char *deviceName)  // IN:
 
    if (ret < 0) {
       Log(LGPFX" Cannot remove lock file %s (%s).\n",
-          path, Err_Errno2String(saveErrno));
+          path, strerror(saveErrno));
       free(path);
 
       return FALSE;
@@ -594,7 +680,7 @@ FileLock_UnlockDevice(const char *deviceName)  // IN:
 /*
  *----------------------------------------------------------------------
  *
- * FileReadSlashProc --
+ * ReadSlashProc --
  *
  *      Read the data in a /proc file
  *
@@ -609,9 +695,9 @@ FileLock_UnlockDevice(const char *deviceName)  // IN:
  */
 
 static int
-FileReadSlashProc(const char *procPath,  // IN:
-                  char *buffer,          // OUT:
-                  size_t bufferSize)     // IN:
+ReadSlashProc(const char *procPath, // IN:
+              char *buffer,         // OUT:
+              size_t bufferSize)    // IN:
 {
    int fd;
    int err;
@@ -650,50 +736,38 @@ FileReadSlashProc(const char *procPath,  // IN:
 
 
 /*
- *---------------------------------------------------------------------------
+ *----------------------------------------------------------------------
  *
- * FileLockProcessDescriptor --
+ * ProcessCreationTime --
  *
- *      Returns the process descriptor of the specified process.
- *
- *      The format of a process descriptor is as follows:
- *
- *      processID-processCreationTime(processName)
- *
- *      where the processName and processCreationTime information
- *      may be independently optional.
+ *      Return the specified process's creation time.
  *
  * Results:
- *      NULL The process does not exist.
- *     !NULL The process descriptor is returned. It is the callers
- *           responsibility to free the dynamically allocated memory.
+ *      The process's creation time is returned. If an error occurs the
+ *      reported creation time will be 0.
  *
- * Side Effects:
- *     None
+ * Side effects:
+ *      None.
  *
- *---------------------------------------------------------------------------
+ *----------------------------------------------------------------------
  */
 
-static char *
-FileLockProcessDescriptor(pid_t pid)  // IN:
+static uint64
+ProcessCreationTime(pid_t pid)
 {
-   char path[64];
-   char buffer[1024];
-   char *descriptor = NULL;
-
-   if (!FileLockIsValidProcess(pid)) {
-      return NULL;
-   }
+   int    err;
+   char   path[64];
+   char   buffer[1024];
+   uint64 creationTime;
 
    Str_Sprintf(path, sizeof path, "/proc/%d/stat", pid);
 
-   if (FileReadSlashProc(path, buffer, sizeof buffer) == 0) {
-      char *p;
-      char *q;
-      char *rest;
-      uint32 argc;
-      char *argv[22];
-      char *savePtr = NULL;
+   err = ReadSlashProc(path, buffer, sizeof buffer);
+
+   if (err == 0) {
+      uint32 i;
+      char   *p;
+      char   *last = NULL;
 
       /*
        * You are in a maze of twisty little fields, (virtually) all alike...
@@ -702,272 +776,78 @@ FileLockProcessDescriptor(pid_t pid)  // IN:
        *
        * A "man 5 proc" will provide illumination concerning all of the
        * fields found on this line of text. We code for the worst case
-       * and ensure that file names containing spaces or parens are
+       * and insure that file names containing spaces or parens are
        * properly handled.
        */
 
-      p = strchr(buffer, '(');
+      p = strrchr(buffer, ')');
+      ASSERT(p != NULL);
+      p = strtok_r(++p, " ", &last);
+      ASSERT(p != NULL);
 
-      if ((p == NULL) || (p == buffer) || (*(p - 1) != ' ')) {
-         goto bail;
+      for (i = 0; i < 19; i++) {
+         p = strtok_r(NULL, " ", &last);
+         ASSERT(p != NULL);
       }
 
-      *(p - 1) = '\0';
+      if (sscanf(p, "%"FMT64"u", &creationTime) != 1) {
+         Warning(LGPFX" %s creationTime conversion error on %s.\n",
+                 __FUNCTION__, p);
 
-      q = strrchr(p + 1, ')');
-      if (q == NULL) {
-         argc = 0;
-      }
-
-      rest = q + 1;
-      if (*rest != ' ') {
-         goto bail;
-      }
-
-      *rest++ = '\0';
-
-      argv[0] = strtok_r(buffer, " ", &savePtr);  // ensure no trailing spaces
-      argv[1] = p;
-
-      /* Map spaces in the process name to something benign */
-      q = p;
-
-      while ((q = strchr(q, ' ')) != NULL) {
-         *q = '_';
-      }
-
-      if (strlen(p) > LOG_MAX_PROC_NAME) {
-         p[LOG_MAX_PROC_NAME - 1] = ')';
-         p[LOG_MAX_PROC_NAME] = '\0';
-      }
-
-      for (argc = 2; argc < 22; argc++) {
-         argv[argc] = strtok_r((argc == 2) ? rest : NULL, " ", &savePtr);
-
-         if (argv[argc] == NULL) {
-            break;
-         }
-      }
-
-      if (argc == 22) {
-         descriptor = Str_SafeAsprintf(NULL, "%s-%s%s", argv[0], argv[21],
-                                       argv[1]);
+         creationTime = 0;
       }
+   } else {
+      creationTime = 0;
    }
 
-bail:
-
-   if (descriptor == NULL) {
-      /*
-       * Accessing /proc failed in some way. Emit a valid string that also
-       * provides a clue that there is/was a problem.
-       */
-
-      descriptor = Str_SafeAsprintf(NULL, "%d-0", pid);
-   }
-
-   return descriptor;
+   return creationTime;
 }
 #elif defined(__APPLE__)
-/*
- *---------------------------------------------------------------------------
- *
- * FileLockProcessCreationTime --
- *
- *      Returns the process creation time of the specified process.
- *
- * Results:
- *      TRUE  Done!
- *      FALSE Process doesn't exist
- *
- * Side effects:
- *      None
- *
- *---------------------------------------------------------------------------
- */
-
-static Bool
-FileLockProcessCreationTime(pid_t pid,                 // IN:
-                            uint64 *procCreationTime)  // OUT:
+static uint64
+ProcessCreationTime(pid_t pid)
 {
-   int err;
-   size_t size;
-   struct kinfo_proc info;
-   int mib[4];
+    int                err;
+    size_t             size;
+    struct kinfo_proc  info;
+    int                mib[4];
+  
+    /* Request information about the specified process */
+    mib[0] = CTL_KERN;
+    mib[1] = KERN_PROC;
+    mib[2] = KERN_PROC_PID;
+    mib[3] = pid;
 
-   ASSERT(procCreationTime);
+    memset(&info, 0, sizeof info);
+    size = sizeof(info);
+    err = sysctl(mib, ARRAYSIZE(mib), &info, &size, NULL, 0);
 
-   /* Request information about the specified process */
-   mib[0] = CTL_KERN;
-   mib[1] = KERN_PROC;
-   mib[2] = KERN_PROC_PID;
-   mib[3] = pid;
+    /* Log any failures */
+    if (err == -1) {
+       Warning(LGPFX" %s sysctl for pid %d failed: %s\n", __FUNCTION__,
+               pid, strerror(errno));
 
-   memset(&info, 0, sizeof info);
-   size = sizeof info;
-   err = sysctl(mib, ARRAYSIZE(mib), &info, &size, NULL, 0);
+       return 0;
+    }
 
-   if (err == -1) {
-      return FALSE;
-   }
-
-   *procCreationTime = (info.kp_proc.p_starttime.tv_sec * CONST64U(1000000)) +
-                        info.kp_proc.p_starttime.tv_usec;
-
-   return TRUE;
-}
-
-
-/*
- *---------------------------------------------------------------------------
- *
- * FileLockProcessDescriptor --
- *
- *      Returns the process descriptor of the specified process.
- *
- *      The format of a process descriptor is as follows:
- *
- *      processID-processCreationTime(processName)
- *
- *      where the processName and processCreationTime information
- *      may be independently optional.
- *
- * Results:
- *      NULL The process does not exist.
- *     !NULL The process descriptor is returned. It is the callers
- *           responsibility to free the dynamically allocated memory.
- *
- * Side effects:
- *      None
- *
- *---------------------------------------------------------------------------
- */
-
-static char *
-FileLockProcessDescriptor(pid_t pid)  // IN:
-{
-   uint64 procCreationTime;
-
-   if (!FileLockIsValidProcess(pid)) {
-      return NULL;
-   }
-
-   if (!FileLockProcessCreationTime(pid, &procCreationTime)) {
-      return NULL;
-   }
-
-   return Str_SafeAsprintf(NULL, "%d-%"FMT64"u", pid, procCreationTime);
+    /* Return the process creation time */
+    return (info.kp_proc.p_starttime.tv_sec * CONST64U(1000000)) +
+            info.kp_proc.p_starttime.tv_usec;
 }
 #else
-/*
- *---------------------------------------------------------------------------
- *
- * FileLockProcessDescriptor --
- *
- *      Returns the process descriptor of the specified process.
- *
- *      The format of a process descriptor is as follows:
- *
- *      processID-processCreationTime(processName)
- *
- *      where the processName and processCreationTime information
- *      may be independently optional.
- *
- * Results:
- *      NULL The process does not exist.
- *     !NULL The process descriptor is returned. It is the callers
- *           responsibility to free the dynamically allocated memory.
- *
- * Side effects:
- *      None
- *
- *---------------------------------------------------------------------------
- */
-
-static char *
-FileLockProcessDescriptor(pid_t pid)  // IN:
+static uint64
+ProcessCreationTime(pid_t pid)
 {
-   return FileLockIsValidProcess(pid) ? Str_SafeAsprintf(NULL, "%d-0", pid) :
-                                        NULL;
+   return 0;
 }
 #endif
 
 
-/*
- *---------------------------------------------------------------------------
- *
- * FileLockGetExecutionID --
- *
- *      Returns the executionID of the caller.
- *
- * Results:
- *      The executionID of the caller. This is a dynamically allocated string;
- *      the caller is responsible for its disposal.
- *
- * Side effects:
- *      The executionID of the caller is not thread safe. Locking is currently
- *      done at the process level - all threads of a process are treated
- *      identically.
- *
- *---------------------------------------------------------------------------
- */
-
-char *
-FileLockGetExecutionID(void)
-{
-   char *descriptor = FileLockProcessDescriptor(getpid());
-
-   ASSERT(descriptor);  // Must be able to describe ourselves!
-
-   return descriptor;
-}
-
-
-/*
- *---------------------------------------------------------------------------
- *
- * FileLockParseProcessDescriptor --
- *
- *      Attempt to parse the specified process descriptor. Return the
- *      pieces requested.
- *
- * Results:
- *      TRUE  Process descriptor is valid.
- *      FALSE Process descriptor is invalid.
- *
- * Side effects:
- *      None
- *
- *---------------------------------------------------------------------------
- */
-
-static Bool
-FileLockParseProcessDescriptor(const char *procDescriptor,  // IN:
-                               pid_t *pid,                  // OUT:
-                               uint64 *procCreationTime)    // OUT:
-{
-   ASSERT(procDescriptor);
-   ASSERT(pid);
-   ASSERT(procCreationTime);
-
-   if (sscanf(procDescriptor, "%d-%"FMT64"u", pid, procCreationTime) != 2) {
-      if (sscanf(procDescriptor, "%d", pid) == 1) {
-         *procCreationTime = 0ULL;
-      } else {
-         return FALSE;
-      }
-   }
- 
-   return *pid >= 0;  
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
- * FileLockValidExecutionID --
+ * FileLockValidOwner --
  *
- *      Validate the execution ID.
+ *      Validate the lock file owner.
  *
  * Results:
  *      TRUE    Yes
@@ -980,47 +860,210 @@ FileLockParseProcessDescriptor(const char *procDescriptor,  // IN:
  */
 
 Bool
-FileLockValidExecutionID(const char *executionID)  // IN:
+FileLockValidOwner(const char *executionID, // IN:
+                   const char *payload)     // IN:
 {
-   pid_t filePID;
-   pid_t procPID;
-   Bool gotFileData;
-   Bool gotProcData;
-   char *procDescriptor;
-   uint64 fileCreationTime;
-   uint64 procCreationTime;
+   int pid;
 
-   gotFileData = FileLockParseProcessDescriptor(executionID, &filePID,
-                                                &fileCreationTime);
-
-   if (!gotFileData) {
-      Warning(LGPFX" %s parse error on '%s'. Assuming valid.\n",
+   /* Validate the PID. */
+   if (sscanf(executionID, "%d", &pid) != 1) {
+      Warning(LGPFX" %s pid conversion error on %s. Assuming valid.\n",
               __FUNCTION__, executionID);
 
-      return TRUE;  // Assume TRUE - preserve a lock - on parse error
+      return TRUE;
    }
 
-   procDescriptor = FileLockProcessDescriptor(filePID);
-
-   if (procDescriptor == NULL) {
-      return FALSE;  // process doesn't exist
+   if (!FileLockIsValidProcess(pid)) {
+      return FALSE;
    }
 
-   gotProcData = FileLockParseProcessDescriptor(procDescriptor, &procPID,
-                                                &procCreationTime);
+   /* If there is a payload perform additional validation. */
+   if (payload != NULL) {
+      uint64 fileCreationTime;
+      uint64 processCreationTime;
 
-   ASSERT(gotProcData);         // We built it; it had better be good
-   ASSERT(procPID == filePID);  // This better match what we started with...
+      /*
+       * The payload is the process creation time of the process that
+       * created the lock file.
+       */
 
-   free(procDescriptor);
+      if (sscanf(payload, "%"FMT64"u", &fileCreationTime) != 1) {
+         Warning(LGPFX" %s payload conversion error on %s. Assuming valid.\n",
+                 __FUNCTION__, payload);
 
-   if ((fileCreationTime != 0) &&
-       (procCreationTime != 0) &&
-       (fileCreationTime != procCreationTime)) {
-      return FALSE;  // The process no longer exists
+         return TRUE;
+      }
+
+      /* Non-matching process creation times -> pid is not the creator */
+      processCreationTime = ProcessCreationTime(pid);
+
+      if ((fileCreationTime != 0) &&
+          (processCreationTime != 0) &&
+          (fileCreationTime != processCreationTime)) {
+         return FALSE;
+      }
+   }
+
+   return TRUE;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ *  FileLockOpenFile --
+ *
+ *      Open the specified file
+ *
+ * Results:
+ *      0       success
+ *      > 0     failure (errno)
+ *
+ * Side effects:
+ *      May change the host file system.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+FileLockOpenFile(ConstUnicode pathName,        // IN:
+                 int flags,                    // IN:
+                 FILELOCK_FILE_HANDLE *handle) // OUT:
+{
+   ASSERT(pathName);
+
+   *handle = PosixFileOpener(pathName, flags, 0644);
+
+   return (*handle == -1) ? errno : 0;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ *  FileLockCloseFile --
+ *
+ *      Close the specified file
+ *
+ * Results:
+ *      0       success
+ *      > 0     failure (errno)
+ *
+ * Side effects:
+ *      May change the host file system.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+FileLockCloseFile(FILELOCK_FILE_HANDLE handle) // IN:
+{
+   return (close(handle) == -1) ? errno : 0;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * FileLockReadFile --
+ *
+ *      Read a file.
+ *
+ * Results:
+ *      0       success
+ *      > 0     failure (errno)
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+FileLockReadFile(FILELOCK_FILE_HANDLE handle,  // IN:
+                 void *buf,                    // IN:
+                 uint32 requestedBytes,        // IN:
+                 uint32 *resultantBytes)       // OUT:
+{
+   int err;
+   ssize_t result;
+
+   result = read(handle, buf, requestedBytes);
+
+   if (result == -1) {
+      *resultantBytes = 0;
+      err = errno;
    } else {
-      return TRUE;  // Looks valid...
+      *resultantBytes = result;
+      err = 0;
    }
+
+   return err;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * FileLockWriteFile --
+ *
+ *      Write a file.
+ *
+ * Results:
+ *      0       success
+ *      > 0     failure (errno)
+ *
+ * Side effects:
+ *      May change the host file system.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+FileLockWriteFile(FILELOCK_FILE_HANDLE handle,  // IN:
+                  void *buf,                    // IN:
+                  uint32 requestedBytes,        // IN:
+                  uint32 *resultantBytes)       // OUT:
+{
+   int err;
+   ssize_t result;
+
+   result = write(handle, buf, requestedBytes);
+
+   if (result == -1) {
+      *resultantBytes = 0;
+      err = errno;
+   } else {
+      *resultantBytes = result;
+      err = 0;
+   }
+
+   return err;
+}
+
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * FileLockGetExecutionID --
+ *
+ *      Returns the executionID of the caller.
+ *
+ * Results:
+ *      The executionID of the caller.
+ *
+ * Side effects:
+ *      The executionID of the caller is not thread safe. Locking is currently
+ *      done at the process level - all threads of a process are treated
+ *      identically.
+ *
+ *---------------------------------------------------------------------------
+ */
+
+char *
+FileLockGetExecutionID(void)
+{
+   return Str_SafeAsprintf(NULL, "%d", FileLockGetPid());
 }
 
 
@@ -1034,7 +1077,7 @@ FileLockValidExecutionID(const char *executionID)  // IN:
  *      points to.
  *
  * Results:
- *      The normalized path or NULL on error
+ *      The normalized path
  *
  * Side effects:
  *      None
@@ -1046,28 +1089,41 @@ static Unicode
 FileLockNormalizePath(ConstUnicode filePath)  // IN:
 {
    Unicode result;
-   Unicode fullPath;
+   struct stat statbuf;
 
-   Unicode dirName = NULL;
-   Unicode fileName = NULL;
+   if ((Posix_Lstat(filePath, &statbuf) == 0) &&
+        (S_ISLNK(statbuf.st_mode))) {
+      Unicode fullPath;
+      Unicode baseDir = NULL;
+      Unicode fileName = NULL;
 
-   /*
-    * If the file to be locked is a symbolic link the lock file belongs next
-    * to the symbolic link, not "off" where the symbolic link points to.
-    * Translation: Don't "full path" the entire path of the file to be locked;
-    * "full path" the dirName of the path, leaving the fileName alone.
-    */
+      /*
+       * The file to be locked is a symbolic path. Place the lock file
+       * next to the file, not where the symbolic path points to.
+       */
 
-   File_GetPathName(filePath, &dirName, &fileName);
+       File_GetPathName(filePath, &baseDir, &fileName);
 
-   fullPath = File_FullPath(dirName);
+       fullPath = File_FullPath(baseDir);
 
-   result = (fullPath == NULL) ? NULL : Unicode_Join(fullPath, DIRSEPS,
-                                                     fileName, NULL);
+       if (fullPath == NULL) {
+          result = NULL;
+       } else {
+          result = Unicode_Join(fullPath, DIRSEPS, fileName, NULL);
 
-   Unicode_Free(fullPath);
-   Unicode_Free(dirName);
-   Unicode_Free(fileName);
+          Unicode_Free(fullPath);
+       }
+
+       Unicode_Free(baseDir);
+       Unicode_Free(fileName);
+   } else {
+      /*
+       * The stat failed or the file is not a symbolic link - do a fullpath
+       * on the argument path to normalize it.
+       */
+
+      result = File_FullPath(filePath);
+   }
 
    return result;
 }
@@ -1100,14 +1156,15 @@ FileLockNormalizePath(ConstUnicode filePath)  // IN:
  *----------------------------------------------------------------------
  */
 
-FileLockToken *
+void *
 FileLock_Lock(ConstUnicode filePath,         // IN:
               const Bool readOnly,           // IN:
               const uint32 msecMaxWaitTime,  // IN:
               int *err)                      // OUT:
 {
+   void *lockToken;
    Unicode normalizedPath;
-   FileLockToken *tokenPtr;
+   char creationTimeString[32];
 
    ASSERT(filePath);
    ASSERT(err);
@@ -1116,15 +1173,18 @@ FileLock_Lock(ConstUnicode filePath,         // IN:
    if (normalizedPath == NULL) {
       *err = EINVAL;
 
-      tokenPtr = NULL;
-   } else {
-      tokenPtr = FileLockIntrinsic(normalizedPath, !readOnly, msecMaxWaitTime,
-                                   err);
-
-      Unicode_Free(normalizedPath);
+      return NULL;
    }
 
-   return tokenPtr;
+   Str_Sprintf(creationTimeString, sizeof creationTimeString, "%"FMT64"u",
+               ProcessCreationTime(FileLockGetPid()));
+
+   lockToken = FileLockIntrinsic(normalizedPath, !readOnly, msecMaxWaitTime,
+                                 creationTimeString, err);
+
+   Unicode_Free(normalizedPath);
+
+   return lockToken;
 }
 
 
@@ -1160,13 +1220,13 @@ FileLock_IsLocked(ConstUnicode filePath,  // IN:
          *err = EINVAL;
       }
 
-      isLocked = FALSE;
-   } else {
-      isLocked = FileLockIsLocked(normalizedPath, err);
-
-      Unicode_Free(normalizedPath);
+      return FALSE;
    }
 
+   isLocked = FileLockIsLocked(normalizedPath, err);
+
+   Unicode_Free(normalizedPath);
+
    return isLocked;
 }
 
@@ -1189,23 +1249,123 @@ FileLock_IsLocked(ConstUnicode filePath,  // IN:
  */
 
 int
-FileLock_Unlock(const FileLockToken *lockToken)  // IN:
+FileLock_Unlock(ConstUnicode filePath,  // IN:
+                const void *lockToken)  // IN:
 {
+   int err;
+   Unicode normalizedPath;
+
+   ASSERT(filePath);
    ASSERT(lockToken);
 
-   return FileUnlockIntrinsic((FileLockToken *) lockToken);
+   normalizedPath = FileLockNormalizePath(filePath);
+   if (normalizedPath == NULL) {
+      return EINVAL;
+   }
+
+   err = FileUnlockIntrinsic(normalizedPath, lockToken);
+
+   Unicode_Free(normalizedPath);
+
+   return err;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * FileLock_DeleteFileVMX --
+ *
+ *      The VMX file delete primitive.
+ *
+ * Results:
+ *      0       unlocked
+ *      >0      errno
+ *
+ * Side effects:
+ *      Changes the host file system.
+ *
+ * Note:
+ *      THIS IS A HORRIBLE HACK AND NEEDS TO BE REMOVED ASAP!!!
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+FileLock_DeleteFileVMX(ConstUnicode filePath)  // IN:
+{
+   int err;
+   Unicode normalizedPath;
+
+   ASSERT(filePath);
+
+   normalizedPath = FileLockNormalizePath(filePath);
+   if (normalizedPath == NULL) {
+      return EINVAL;
+   }
+
+   err = FileLockHackVMX(normalizedPath);
+
+   Unicode_Free(normalizedPath);
+
+   return err;
 }
 
 #else
+
+/*
+ * Stub functions for unsupported platforms.
+ */
+ 
+int
+FileLockCloseFile(FILELOCK_FILE_HANDLE handle)  // IN:
+{
+   NOT_IMPLEMENTED();
+}
+
+
 char *
 FileLockGetExecutionID(void)
 {
    NOT_IMPLEMENTED();
 }
 
-Bool
-FileLockValidExecutionID(const char *executionID)  // IN:
+
+int
+FileLockOpenFile(ConstUnicode pathName,         // IN:
+                 int flags,                     // IN:
+                 FILELOCK_FILE_HANDLE *handle)  // OUT:
+{
+
+   NOT_IMPLEMENTED();
+}
+
+
+int
+FileLockReadFile(FILELOCK_FILE_HANDLE handle,  // IN:
+                 void *buf,                    // IN:
+                 uint32 requestedBytes,        // IN:
+                 uint32 *resultantBytes)       // OUT:
 {
    NOT_IMPLEMENTED();
 }
+
+
+Bool
+FileLockValidOwner(const char *executionID,  // IN:
+                   const char *payload)      // IN:
+{
+   NOT_IMPLEMENTED();
+}
+
+
+int
+FileLockWriteFile(FILELOCK_FILE_HANDLE handle,  // IN:
+                  void *buf,                    // IN:
+                  uint32 requestedBytes,        // IN:
+                  uint32 *resultantBytes)       // OUT:
+{
+   NOT_IMPLEMENTED();
+}
+
 #endif /* !__FreeBSD__ && !sun */
