@@ -48,7 +48,7 @@
  */
 
 void
-DynBuf_Init(DynBuf *b)  // OUT:
+DynBuf_Init(DynBuf *b) // IN
 {
    ASSERT(b);
 
@@ -75,12 +75,13 @@ DynBuf_Init(DynBuf *b)  // OUT:
  */
 
 void
-DynBuf_Destroy(DynBuf *b)  // IN/OUT:
+DynBuf_Destroy(DynBuf *b) // IN
 {
    ASSERT(b);
 
    free(b->data);
-   DynBuf_Init(b);
+   b->data = NULL;
+   b->allocated = 0;
 }
 
 
@@ -338,38 +339,6 @@ DynBuf_Append(DynBuf *b,        // IN
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * DynBuf_SafeInternalAppend --
- *
- *      Append data at the end of a dynamic buffer. Memory allocation failure
- *      are handled the same way as Util_SafeMalloc, that is to say, with a
- *      Panic.
- *
- * Results:
- *      None
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-DynBuf_SafeInternalAppend(DynBuf *b,            // IN
-                          void const *data,     // IN
-                          size_t size,          // IN
-                          char const *file,     // IN
-                          unsigned int lineno)  // IN
-{
-   if (!DynBuf_Append(b, data, size)) {
-      Panic("Unrecoverable memory allocation failure at %s:%u\n",
-            file, lineno);
-   }
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
