@@ -33,8 +33,6 @@
 #include <unistd.h>
 #if !defined(__FreeBSD__) && !defined(sun) && !defined(__APPLE__)
 #include <asm/param.h>
-#endif
-#if !defined(sun) && !defined(__APPLE__)
 #include <locale.h>
 #include <sys/stat.h>
 #endif
@@ -50,18 +48,13 @@
 #include <time.h>
 #include <grp.h>
 #include <sys/syscall.h>
-#if defined(linux) || defined(__FreeBSD__) || defined(HAVE_SYS_USER_H)
+#if defined(linux) || defined(HAVE_SYS_USER_H)
 // sys/param.h is required on FreeBSD before sys/user.h
 #   include <sys/param.h>
 // Pull in PAGE_SIZE/PAGE_SHIFT defines ahead of vm_basic_defs.h
 #   include <sys/user.h>
 #endif
-#if defined (__FreeBSD__)
-#include <kvm.h>
-#include <limits.h>
-#include <paths.h>
-#include <sys/sysctl.h>
-#endif
+
 #include "vmware.h"
 #include "procMgr.h"
 #include "vm_assert.h"
@@ -106,8 +99,7 @@ struct ProcMgr_AsyncProc {
 };
 
 static pid_t ProcMgrStartProcess(char const *cmd,
-                                 char * const  *envp,
-                                 char const *workingDir);
+                                 char * const  *envp);
 
 static Bool ProcMgrWaitForProcCompletion(pid_t pid,
                                          Bool *validExitCode,
@@ -117,10 +109,6 @@ static Bool ProcMgrKill(pid_t pid,
                         int sig,
                         int timeout);
 
-#ifdef sun
-#define  SOLARIS_BASH_PATH "/usr/bin/bash"
-#endif
-
 #if defined(linux) && !defined(GLIBC_VERSION_23)
 /*
  * Implements the system calls (they are not wrapped by glibc til 2.3.2).
@@ -150,92 +138,6 @@ setresgid(gid_t ruid,
 #endif
 
 
-/*
- *----------------------------------------------------------------------
- *
- * ProcMgr_ReadProcFile --
- *
- *    Read the contents of a file in /proc/<pid>.
- *
- *    The size is essentially unbounded because of cmdline arguments.
- *    The only way to figure out the content size is to keep reading;
- *    stat(2) and lseek(2) lie.
- *
- *    The contents are NUL terminated -- in some distros may not include
- *    a NUL for some commands (eg. pid 1, /sbin/init) -- so add
- *    one to be safe.
- *
- * Results:
- *
- *    The length of the file.
- *
- *    -1 on error.
- *
- * Side effects:
- *
- *    The returned contents must be freed by caller.
- *
- *----------------------------------------------------------------------
- */
-
-#if !defined(sun) && !defined(__FreeBSD__)
-int
-ProcMgr_ReadProcFile(int fd,                       // IN
-                     char **contents)              // OUT
-{
-   int size = 0;
-#if !defined(__APPLE__)
-   char tmp[512];
-   int numRead;
-
-   *contents = NULL;
-   numRead = read(fd, tmp, sizeof(tmp));
-   size = numRead;
-
-   if (numRead <= 0) {
-      goto done;
-   }
-
-   /*
-    * handle the 99% case
-    */
-   if (sizeof(tmp) > numRead) {
-      char *result;
-
-      result = malloc(numRead + 1);
-      if (NULL == result) {
-         size = -1;
-         goto done;
-      }
-      memcpy(result, tmp, numRead);
-      result[numRead] = '\0';
-      *contents = result;
-      goto done;
-   } else {
-      DynBuf dbuf;
-
-      DynBuf_Init(&dbuf);
-      DynBuf_Append(&dbuf, tmp, numRead);
-      do {
-         numRead = read(fd, tmp, sizeof(tmp));
-         if (numRead > 0) {
-            DynBuf_Append(&dbuf, tmp, numRead);
-         }
-         size += numRead;
-      } while (numRead > 0);
-      // add the NUL term
-      DynBuf_Append(&dbuf, "", 1);
-      DynBuf_Trim(&dbuf);
-      *contents = DynBuf_Detach(&dbuf);
-      DynBuf_Destroy(&dbuf);
-   }
-done:
-#endif
-   return size;
-}
-
-#endif   // !sun && !FreeBSD
-
 /*
  *----------------------------------------------------------------------
  *
@@ -254,12 +156,11 @@ done:
  *----------------------------------------------------------------------
  */
 
-#if !defined(sun) && !defined(__FreeBSD__)
 ProcMgr_ProcList *
 ProcMgr_ListProcesses(void)
 {
    ProcMgr_ProcList *procList = NULL;
-#if !defined(__APPLE__)
+#if !defined(__FreeBSD__) && !defined(sun) && !defined(__APPLE__)
    Bool failed = FALSE;
    DynBuf dbProcId;
    DynBuf dbProcCmd;
@@ -343,8 +244,8 @@ ProcMgr_ListProcesses(void)
       pid_t pid;
       int replaceLoop;
       struct passwd *pwd;
-      char *cmdLineTemp = NULL;
-      char *cmdStatTemp = NULL;
+      char cmdLineTemp[2048];
+      char cmdStatTemp[2048];
       char *cmdLine;
       char *userName = NULL;
       size_t strLen = 0;
@@ -389,18 +290,14 @@ ProcMgr_ListProcesses(void)
        * In the future, we could keep the NUL version around and pass it
        * back to the client for easier parsing when retrieving individual
        * command line parameters is needed.
+       *
+       * We read at most (sizeof cmdLineTemp) - 1 bytes to leave room
+       * for NUL termination at the end.
        */
-      numRead = ProcMgr_ReadProcFile(cmdFd, &cmdLineTemp);
+      numRead = read(cmdFd, cmdLineTemp, sizeof cmdLineTemp - sizeof(char));
       close(cmdFd);
 
-      if (numRead < 0) {
-         continue;
-      }
-
       if (numRead > 0) {
-         /*
-          * Stop before we hit the final '\0'; want to leave it alone.
-          */
          for (replaceLoop = 0 ; replaceLoop < (numRead - 1) ; replaceLoop++) {
             if ('\0' == cmdLineTemp[replaceLoop]) {
                cmdLineTemp[replaceLoop] = ' ';
@@ -423,11 +320,17 @@ ProcMgr_ListProcesses(void)
             cmdFd = open(cmdFilePath, O_RDONLY);
          }
          if (cmdFd != -1) {
-            numRead = ProcMgr_ReadProcFile(cmdFd, &cmdLineTemp);
+            numRead = read(cmdFd, cmdLineTemp, sizeof(cmdLineTemp) - sizeof(char));
             close(cmdFd);
+
+            if (numRead < 0) {
+               cmdLineTemp[0] = '\0';
+            } else {
+               cmdLineTemp[numRead] = '\0';
+            }
          }
          if (numRead > 0) {
-            /*
+            /* 
              * Extract the part with just the name, by reading until the first
              * space, then reading the next non-space word after that, and
              * ignoring everything else. The format looks like this:
@@ -435,8 +338,8 @@ ProcMgr_ListProcesses(void)
              * for example:
              *     "Name:    nfsd"
              */
-            const char *nameStart;
-            char *copyItr;
+            const char* nameStart = NULL;
+            char* copyItr = NULL;
 
             /* Skip non-whitespace. */
             for (nameStart = cmdLineTemp; *nameStart && 
@@ -456,6 +359,21 @@ ProcMgr_ListProcesses(void)
          }
       }
 
+      /*
+       * There is an edge case where /proc/#/cmdline does not NUL terminate
+       * the command.  /sbin/init (process 1) is like that on some distros.
+       * So let's guarantee that the string is NUL terminated, even if
+       * the last character of the string might already be NUL.
+       * This is safe to do because we read at most (sizeof cmdLineTemp) - 1
+       * bytes from /proc/#/cmdline -- we left just enough space to add
+       * NUL termination at the end.
+       */
+      if (numRead < 0) {
+         cmdLineTemp[0] = '\0';
+      } else {
+         cmdLineTemp[numRead] = '\0';
+      }
+
       /*
        * Get the inode information for this process.  This gives us
        * the process owner.
@@ -465,7 +383,7 @@ ProcMgr_ListProcesses(void)
                    "/proc/%s",
                    ent->d_name) == -1) {
          Debug("Giant process id '%s'\n", ent->d_name);
-         goto next_entry;
+         continue;
       }
 
       /*
@@ -475,7 +393,7 @@ ProcMgr_ListProcesses(void)
        */
       statResult = stat(cmdFilePath, &fileStat);
       if (0 != statResult) {
-         goto next_entry;
+         continue;
       }
 
       /*
@@ -487,22 +405,22 @@ ProcMgr_ListProcesses(void)
                    "/proc/%s/stat",
                    ent->d_name) == -1) {
          Debug("Giant process id '%s'\n", ent->d_name);
-         goto next_entry;
+         continue;
       }
       cmdFd = open(cmdFilePath, O_RDONLY);
       if (-1 == cmdFd) {
-         goto next_entry;
+         continue;
       }
-      numRead = ProcMgr_ReadProcFile(cmdFd, &cmdStatTemp);
+      numRead = read(cmdFd, cmdStatTemp, sizeof cmdStatTemp);
       close(cmdFd);
       if (0 >= numRead) {
-         goto next_entry;
+         continue;
       }
       /*
        * Skip over initial process id and process name.  "123 (bash) [...]".
        */
       stringBegin = strchr(cmdStatTemp, ')') + 2;
-
+      
       numberFound = sscanf(stringBegin, "%c %d %d %d %d %d "
                            "%lu %lu %lu %lu %lu %Lu %Lu %Lu %Lu %ld %ld "
                            "%d %ld %Lu",
@@ -519,18 +437,14 @@ ProcMgr_ListProcesses(void)
                            (int *) &dummy, (long *) &dummy,
                            &relativeStartTime);
       if (20 != numberFound) {
-         goto next_entry;
+         continue;
       }
       processStartTime = hostStartTime + (relativeStartTime / hertz);
 
       /*
        * Store the command line string pointer in dynbuf.
        */
-      if (cmdLineTemp) {
-         cmdLine = Unicode_Alloc(cmdLineTemp, STRING_ENCODING_DEFAULT);
-      } else {
-         cmdLine = Unicode_Alloc("", STRING_ENCODING_UTF8);
-      }
+      cmdLine = strdup(cmdLineTemp);
       DynBuf_Append(&dbProcCmd, &cmdLine, sizeof cmdLine);
 
       /*
@@ -545,7 +459,7 @@ ProcMgr_ListProcesses(void)
       pwd = getpwuid(fileStat.st_uid);
       userName = (NULL == pwd)
                  ? Str_Asprintf(&strLen, "%d", (int) fileStat.st_uid)
-                 : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
+                 : Util_SafeStrdup(pwd->pw_name);
       DynBuf_Append(&dbProcOwner, &userName, sizeof userName);
 
       /*
@@ -554,10 +468,6 @@ ProcMgr_ListProcesses(void)
       DynBuf_Append(&dbProcStartTime,
                     &processStartTime,
                     sizeof processStartTime);
-
-next_entry:
-      free(cmdLineTemp);
-      free(cmdStatTemp);
    } // while readdir
 
    closedir(dir);
@@ -602,212 +512,12 @@ abort:
       ProcMgr_FreeProcList(procList);
       procList = NULL;
    }
-#endif // !defined(__APPLE__)
+#endif // !defined(__FreeBSD__) && !defined(sun) && !defined(__APPLE__)
 
    return procList;
 }
-#endif // !defined(sun) & !defined(__FreeBSD__)
 
 
-/*
- *----------------------------------------------------------------------
- *
- * ProcMgr_ListProcesses --
- *
- *      List all the processes that the calling client has privilege to
- *      enumerate. The strings in the returned structure should be all
- *      UTF-8 encoded, although we do not enforce it right now.
- *
- * Results:
- *
- *      A ProcMgr_ProcList.
- *
- * Side effects:
- *
- *----------------------------------------------------------------------
- */
-
-#if defined(__FreeBSD__)
-ProcMgr_ProcList *
-ProcMgr_ListProcesses(void)
-{
-   ProcMgr_ProcList *procList = NULL;
-   Bool failed = TRUE;
-   static kvm_t *kd;
-   struct kinfo_proc *kp;
-   char errbuf[_POSIX2_LINE_MAX];
-   int i, nentries=-1, flag=0;
-   DynBuf dbProcId;
-   DynBuf dbProcCmd;
-   DynBuf dbProcStartTime;
-   DynBuf dbProcOwner;
-
-   DynBuf_Init(&dbProcId);
-   DynBuf_Init(&dbProcCmd);
-   DynBuf_Init(&dbProcStartTime);
-   DynBuf_Init(&dbProcOwner);
-
-   /*
-    * Get the handle to the Kernel Virtual Memory
-    */
-   kd = kvm_openfiles(NULL, _PATH_DEVNULL, NULL, O_RDONLY, errbuf);
-   if (kd == NULL) {
-      Warning("%s: failed to open kvm with error: %s\n", __FUNCTION__, errbuf);
-      goto abort;
-   }
-
-   /*
-    * Get the list of process info structs
-    */
-   kp = kvm_getprocs(kd, KERN_PROC_PROC, flag, &nentries);
-   if ((kp == NULL && nentries > 0) || (kp != NULL && nentries < 0)) {
-      Warning("%s: failed to get proc infos with error: %s\n",
-              __FUNCTION__, kvm_geterr(kd));
-      goto abort;
-   }
-
-   /*
-    * Iterate through the list of process entries
-    */
-   for (i = 0; i < nentries; ++i, ++kp) {
-      struct passwd *pwd;
-      char *userName = NULL;
-      char *cmdLine = NULL;
-      char **cmdLineTemp = NULL;
-      time_t processStartTime;
-
-      /*
-       * Store the pid of the process
-       */
-      if (!DynBuf_Append(&dbProcId, &(kp->ki_pid), sizeof(kp->ki_pid))) {
-         Warning("%s: failed to append pid in DynBuf - out of memory\n",
-                 __FUNCTION__);
-         goto abort;
-      }
-
-      /*
-       * Store the owner of the process.
-       */
-      pwd = getpwuid(kp->ki_uid);
-      userName = (NULL == pwd)
-                 ? Str_SafeAsprintf(NULL, "%d", (int) kp->ki_uid)
-                 : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
-      if (!DynBuf_Append(&dbProcOwner, &userName, sizeof userName)) {
-         Warning("%s: failed to append username in DynBuf - out of memory\n",
-                 __FUNCTION__);
-         goto abort;
-      }
-
-      /*
-       * Store the command line string of the process
-       */
-      cmdLineTemp = kvm_getargv(kd, kp, 0);
-      if (cmdLineTemp != NULL) {
-         // flatten the argument list to get cmd & all params
-         DynBuf dbuf;
-
-         DynBuf_Init(&dbuf);
-         while (*cmdLineTemp != NULL) {
-            if (!DynBuf_Append(&dbuf, *cmdLineTemp, strlen(*cmdLineTemp))) {
-               Warning("%s: failed to append cmd/args in DynBuf - no memory\n",
-                       __FUNCTION__);
-               goto abort;
-            }
-            cmdLineTemp++;
-            if (*cmdLineTemp != NULL) {
-               // add the whitespace between arguments
-               if (!DynBuf_Append(&dbuf, " ", 1)) {
-                  Warning("%s: failed to append ' ' in DynBuf - no memory\n",
-                          __FUNCTION__);
-                  goto abort;
-               }
-            }
-         }
-         // add the NUL term
-         if (!DynBuf_Append(&dbuf, "", 1)) {
-            Warning("%s: failed to append NUL in DynBuf - out of memory\n",
-                    __FUNCTION__);
-            goto abort;
-         }
-         DynBuf_Trim(&dbuf);
-         cmdLine = DynBuf_Detach(&dbuf);
-         DynBuf_Destroy(&dbuf);
-      } else {
-         cmdLine = Unicode_Alloc(kp->ki_comm, STRING_ENCODING_DEFAULT);
-      }
-      if (!DynBuf_Append(&dbProcCmd, &cmdLine, sizeof cmdLine)) {
-         Warning("%s: failed to append cmdline in DynBuf - out of memory\n",
-                 __FUNCTION__);
-         goto abort;
-      }
-
-      /*
-       * Store the start time of the process
-       */
-      processStartTime = kp->ki_start.tv_sec;
-      if (!DynBuf_Append(&dbProcStartTime,
-                    &processStartTime,
-                    sizeof processStartTime)) {
-         Warning("%s: failed to append start time in DynBuf - out of memory\n",
-                 __FUNCTION__);
-         goto abort;
-      }
-
-   } // for nentries
-
-   if (0 == DynBuf_GetSize(&dbProcId)) {
-      goto abort;
-   }
-
-   /*
-    * We're done adding to DynBuf.  Trim off any unused allocated space.
-    * DynBuf_Trim() followed by DynBuf_Detach() avoids a memcpy().
-    */
-   DynBuf_Trim(&dbProcId);
-   DynBuf_Trim(&dbProcCmd);
-   DynBuf_Trim(&dbProcStartTime);
-   DynBuf_Trim(&dbProcOwner);
-
-   /*
-    * Create a ProcMgr_ProcList and populate its fields.
-    */
-   procList = (ProcMgr_ProcList *) Util_SafeCalloc(1, sizeof(ProcMgr_ProcList));
-   ASSERT_NOT_IMPLEMENTED(procList);
-
-   procList->procCount = DynBuf_GetSize(&dbProcId) / sizeof(pid_t);
-
-   procList->procIdList = (pid_t *) DynBuf_Detach(&dbProcId);
-   ASSERT_NOT_IMPLEMENTED(procList->procIdList);
-   procList->procCmdList = (char **) DynBuf_Detach(&dbProcCmd);
-   ASSERT_NOT_IMPLEMENTED(procList->procCmdList);
-   procList->startTime = (time_t *) DynBuf_Detach(&dbProcStartTime);
-   ASSERT_NOT_IMPLEMENTED(procList->startTime);
-   procList->procOwnerList = (char **) DynBuf_Detach(&dbProcOwner);
-   ASSERT_NOT_IMPLEMENTED(procList->procOwnerList);
-
-   failed = FALSE;
-abort:
-   DynBuf_Destroy(&dbProcId);
-   DynBuf_Destroy(&dbProcCmd);
-   DynBuf_Destroy(&dbProcStartTime);
-   DynBuf_Destroy(&dbProcOwner);
-
-   if (kd != NULL) {
-      /*
-       * Wrap up: also deallocates memory (if) allocated by kvm_getargv()
-       */
-      kvm_close(kd);
-   }
-
-   if (failed) {
-      ProcMgr_FreeProcList(procList);
-      procList = NULL;
-   }
-
-   return procList;
-}
-#endif // defined(__FreeBSD__)
-
 /*
  *----------------------------------------------------------------------
  *
@@ -871,8 +581,7 @@ ProcMgr_ExecSync(char const *cmd,                  // IN: UTF-8 command line
 
    Debug("Executing sync command: %s\n", cmd);
 
-   pid = ProcMgrStartProcess(cmd, userArgs ? userArgs->envp : NULL,
-                             userArgs ? userArgs->workingDirectory : NULL);
+   pid = ProcMgrStartProcess(cmd, userArgs ? userArgs->envp : NULL);
 
    if (pid == -1) {
       return FALSE;
@@ -901,13 +610,11 @@ ProcMgr_ExecSync(char const *cmd,                  // IN: UTF-8 command line
 
 static pid_t
 ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
-                    char * const *envp,         // IN: UTF-8 encoded env vars
-                    char const *workingDir)     // IN: UTF-8 working directory
+                    char * const *envp)         // IN: UTF-8 encoded env vars
 {
    pid_t pid;
    char *cmdCurrent = NULL;
    char **envpCurrent = NULL;
-   char *workDir = NULL;
 
    if (cmd == NULL) {
       ASSERT(FALSE);
@@ -924,12 +631,6 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
       return -1;
    }
 
-   if ((NULL != workingDir) &&
-       !CodeSet_Utf8ToCurrent(workingDir, strlen(workingDir), &workDir, NULL)) {
-      Warning("Could not convert workingDir from UTF-8 to current\n");
-      return -1;
-   }
-
    if (NULL != envp) {
       envpCurrent = Unicode_GetAllocList(envp, -1, STRING_ENCODING_DEFAULT);
    }
@@ -939,52 +640,13 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
    if (pid == -1) {
       Warning("Unable to fork: %s.\n\n", strerror(errno));
    } else if (pid == 0) {
-#ifdef sun
-      /*
-       * On Solaris, /bin/sh is the Bourne shell, and it
-       * doesn't appear to have the optimization that bash does -- when
-       * called with -c, bash appears to just use exec() to replace itself.
-       * Bourne shell does a fork & exec, so 2 processes are started.
-       * This is bad for us because we then see the PID of the shell, not the
-       * app that it starts.  When this PID is returned to a user to
-       * watch, they'll watch the wrong process.
-       *
-       * So for Solaris, use bash instead if possible.  We support
-       * Solaris 10 and better; it contains bash, but not in its
-       * minimal 'core' package, so it may not exist.
-       */
-      static const char bashShellPath[] = SOLARIS_BASH_PATH;
-      char *bashArgs[] = { "bash", "-c", cmdCurrent, NULL };
-      static const char bourneShellPath[] = "/bin/sh";
-      char *bourneArgs[] = { "sh", "-c", cmdCurrent, NULL };
-      const char *shellPath;
-      char **args;
-#else
       static const char shellPath[] = "/bin/sh";
       char *args[] = { "sh", "-c", cmdCurrent, NULL };
-#endif
-
-#ifdef sun
-      if (File_Exists(SOLARIS_BASH_PATH)) {
-         shellPath = bashShellPath;
-         args = bashArgs;
-      } else {
-         shellPath = bourneShellPath;
-         args = bourneArgs;
-      }
-#endif
 
       /*
        * Child
        */
 
-      if (NULL != workDir) {
-         if (chdir(workDir) != 0) {
-            Warning("%s: Could not chdir(%s) %s\n", __FUNCTION__, workDir,
-                    strerror(errno));
-         }
-      }
-
       if (NULL != envpCurrent) {
          execve(shellPath, args, envpCurrent);
       } else  {
@@ -1001,7 +663,6 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
     */
 
    free(cmdCurrent);
-   free(workDir);
    Unicode_FreeList(envpCurrent, -1);
    return pid;
 }
@@ -1102,8 +763,7 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
    FileIODescriptor readFd;
    FileIODescriptor writeFd;
 
-   Debug("Executing async command: '%s' in working dir '%s'\n",
-         cmd, (userArgs && userArgs->workingDirectory) ? userArgs->workingDirectory : "");
+   Debug("Executing async command: %s\n", cmd);
    
    if (pipe(fds) == -1) {
       Warning("Unable to create the pipe to launch command: %s.\n", cmd);
@@ -1159,9 +819,7 @@ ProcMgr_ExecAsync(char const *cmd,                 // IN: UTF-8 command line
        * Only run the program if we have not already experienced a failure.
        */
       if (status) {
-         childPid = ProcMgrStartProcess(cmd,
-                                        userArgs ? userArgs->envp : NULL,
-                                        userArgs ? userArgs->workingDirectory : NULL);
+         childPid = ProcMgrStartProcess(cmd, userArgs ? userArgs->envp : NULL);
          status = childPid != -1;
       }
 
@@ -1433,6 +1091,64 @@ ProcMgr_Kill(ProcMgr_AsyncProc *asyncProc) // IN
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * ProcMgr_GetAsyncStatus --
+ *
+ *      Get the return status of an async process.
+ *      Must only be called once for any async process.
+ *
+ * Results:
+ *      TRUE if the status was retrieved.
+ *      FALSE if it couldn't be retrieved.
+ *
+ * Side effects:
+ *	Does a waitpid() on the child to prevent zombification. 
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+ProcMgr_GetAsyncStatus(ProcMgr_AsyncProc *asyncProc, // IN
+                       Bool *status)                 // OUT
+{
+   Bool retVal = FALSE;
+  
+   ASSERT(status);
+   ASSERT(asyncProc);
+   ASSERT(asyncProc->waiterPid != -1);
+
+   if (FileIO_Read(&(asyncProc->fd), status, sizeof *status, NULL) !=
+       FILEIO_SUCCESS) {
+	 Warning("Error reading async process status.\n");
+      goto end;
+   }
+
+   if (FileIO_Read(&(asyncProc->fd), &(asyncProc->exitCode), 
+                   sizeof asyncProc->exitCode, NULL) !=
+       FILEIO_SUCCESS) {
+	 Warning("Error reading async process status.\n");
+      goto end;
+   }
+
+   asyncProc->validExitCode = TRUE;
+
+   Debug("Child w/ fd %x exited with status=%d\n", 
+         asyncProc->fd.posix, *status);
+
+   retVal = TRUE;
+
+ end:
+   /* Read the pid so the processes don't become zombied */
+   Debug("Waiting on pid %"FMTPID" to de-zombify it\n", asyncProc->waiterPid);
+   waitpid(asyncProc->waiterPid, NULL, 0);
+   asyncProc->waiterPid = -1;
+
+   return retVal;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1540,14 +1256,13 @@ ProcMgr_GetPid(ProcMgr_AsyncProc *asyncProc)
  *
  * ProcMgr_GetExitCode --
  *
- *      Get the exit code status of an async process. Waits on the child
- *      process so that its resources are cleaned up.
+ *      Get the exit code status of an async process.
  *
  * Results:
  *      0 if successful, -1 if not.
  *
  * Side effects:
- *	     None.
+ *	See ProcMgr_GetAsyncStatus().
  *
  *----------------------------------------------------------------------
  */
@@ -1559,39 +1274,22 @@ ProcMgr_GetExitCode(ProcMgr_AsyncProc *asyncProc,  // IN
    ASSERT(asyncProc);
    ASSERT(exitCode);
 
-   *exitCode = -1;
+   if (!asyncProc->validExitCode) {
+      Bool dummy;
 
-   if (asyncProc->waiterPid != -1) {
-      Bool status;
-
-      if (FileIO_Read(&(asyncProc->fd), &status, sizeof status, NULL) !=
-          FILEIO_SUCCESS) {
-         Warning("Error reading async process status.\n");
-         goto exit;
+      if (!ProcMgr_GetAsyncStatus(asyncProc, &dummy)) {
+         *exitCode = -1;
+         return(-1);
       }
 
-      if (FileIO_Read(&(asyncProc->fd), &(asyncProc->exitCode),
-                      sizeof asyncProc->exitCode, NULL) !=
-          FILEIO_SUCCESS) {
-         Warning("Error reading async process status.\n");
-         goto exit;
+      if (!(asyncProc->validExitCode)) {
+         *exitCode = -1;
+         return(-1);
       }
-
-      asyncProc->validExitCode = TRUE;
-
-      Debug("Child w/ fd %x exited with code=%d\n",
-            asyncProc->fd.posix, asyncProc->exitCode);
    }
 
    *exitCode = asyncProc->exitCode;
-
-exit:
-   if (asyncProc->waiterPid != -1) {
-      Debug("Waiting on pid %"FMTPID" to de-zombify it\n", asyncProc->waiterPid);
-      waitpid(asyncProc->waiterPid, NULL, 0);
-      asyncProc->waiterPid = -1;
-   }
-   return (asyncProc->exitCode == -1) ? -1 : 0;
+   return(0);
 }
 
 
@@ -1641,7 +1339,7 @@ ProcMgr_Free(ProcMgr_AsyncProc *asyncProc) // IN
    free(asyncProc);
 }
 
-#if defined(linux) || defined(__FreeBSD__)
+#ifdef linux
 
 /*
  *----------------------------------------------------------------------
@@ -1794,50 +1492,4 @@ ProcMgr_ImpersonateUserStop(void)
    return TRUE;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * ProcMgr_GetImpersonatedUserInfo --
- *
- *      Return info about the impersonated user.
- *
- * Results:
- *      TRUE on success
- *
- * Side effects:
- *
- *----------------------------------------------------------------------
- */
-
-Bool
-ProcMgr_GetImpersonatedUserInfo(char **userName,            // OUT
-                                char **homeDir)             // OUT
-{
-   uid_t uid = geteuid();
-   char buffer[BUFSIZ];
-   struct passwd pw;
-   struct passwd *ppw = &pw;
-   int error;
-
-   *userName = NULL;
-   *homeDir = NULL;
-   if ((error = getpwuid_r(uid, &pw, buffer, sizeof buffer, &ppw)) != 0 ||
-       !ppw) {
-      /*
-       * getpwuid_r() and getpwnam_r() can return a 0 (success) but not
-       * set the return pointer (ppw) if there's no entry for the user,
-       * according to POSIX 1003.1-2003, so patch up the errno.
-       */
-      if (error == 0) {
-         error = ENOENT;
-      }
-      return FALSE;
-   }
-
-   *userName = Unicode_Alloc(ppw->pw_name, STRING_ENCODING_DEFAULT);
-   *homeDir = Unicode_Alloc(ppw->pw_dir, STRING_ENCODING_DEFAULT);
-
-   return TRUE;
-}
-
-#endif // linux || __FreeBSD__
+#endif // linux
