@@ -24,7 +24,6 @@
 #include <linux/rtnetlink.h>
 #include <linux/netdevice.h>
 #include <linux/etherdevice.h>
-#include <linux/pci.h>
 
 /*
  * The enet_statistics structure moved from linux/if_ether.h to
@@ -196,7 +195,7 @@ compat_alloc_netdev(int priv_size,
 #define netdev_priv(dev)               ((dev)->priv)
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 9)
+#if defined(NETDEV_TX_OK)
 #   define COMPAT_NETDEV_TX_OK    NETDEV_TX_OK
 #   define COMPAT_NETDEV_TX_BUSY  NETDEV_TX_BUSY
 #else
@@ -281,7 +280,7 @@ static inline int compat_unregister_netdevice_notifier(struct notifier_block *nb
         unregister_netdevice_notifier(_nb);
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || defined(__VMKLNX__)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
 
 #   define compat_netif_napi_add(dev, napi, poll, quota) \
       netif_napi_add(dev, napi, poll, quota)
@@ -300,102 +299,25 @@ static inline int compat_unregister_netdevice_notifier(struct notifier_block *nb
 
 #else
 
-#   define compat_napi_complete(dev, napi) netif_rx_complete(dev)
+#   define compat_netif_napi_add(dev, napi, pollcb, quota) \
+   do {                        \
+      (dev)->poll = (pollcb);    \
+      (dev)->weight = (quota);\
+   } while (0)
 #   define compat_napi_schedule(dev, napi) netif_rx_schedule(dev)
 #   define compat_napi_enable(dev, napi)   netif_poll_enable(dev)
 #   define compat_napi_disable(dev, napi)  netif_poll_disable(dev)
 
 /* RedHat ported GRO to 2.6.18 bringing new napi_struct with it */
 #   if defined NETIF_F_GRO
-#      define compat_netif_napi_add(netdev, napi, pollcb, quota) \
-      do {                        \
-         (netdev)->poll = (pollcb);    \
-         (netdev)->weight = (quota);\
-         (napi)->dev = (netdev); \
-      } while (0)
-
+#      define compat_napi_complete(dev, napi) napi_complete(napi)
 #   else
+#      define compat_napi_complete(dev, napi) netif_rx_complete(dev)
        struct napi_struct {
           int dummy;
        };
-#      define compat_netif_napi_add(dev, napi, pollcb, quota) \
-       do {                        \
-          (dev)->poll = (pollcb);    \
-          (dev)->weight = (quota);\
-       } while (0)
-
 #   endif
 
 #endif
 
-#ifdef NETIF_F_TSO6
-#  define COMPAT_NETIF_F_TSO (NETIF_F_TSO6 | NETIF_F_TSO)
-#else
-#  define COMPAT_NETIF_F_TSO (NETIF_F_TSO)
-#endif
-
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
-#   define compat_netif_tx_lock(dev) netif_tx_lock(dev)
-#   define compat_netif_tx_unlock(dev) netif_tx_unlock(dev)
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
-#   define compat_netif_tx_lock(dev) spin_lock(&dev->xmit_lock)
-#   define compat_netif_tx_unlock(dev) spin_unlock(&dev->xmit_lock)
-#else
-/* Vendor backporting (SLES 10) has muddled the tx_lock situation. Pick whichever
- * of the above works for you. */
-#   define compat_netif_tx_lock(dev) do {} while (0)
-#   define compat_netif_tx_unlock(dev) do {} while (0)
-#endif
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 37)
-#   define COMPAT_VLAN_GROUP_ARRAY_LEN VLAN_N_VID
-#   define compat_flush_scheduled_work(work) cancel_work_sync(work)
-#else
-#   define COMPAT_VLAN_GROUP_ARRAY_LEN VLAN_GROUP_ARRAY_LEN
-#   define compat_flush_scheduled_work(work) flush_scheduled_work()
-#endif
-
-
-
-/*
- * For kernel versions older than 2.6.29, where pci_msi_enabled is not
- * available, check if
- *	1. CONFIG_PCI_MSI is present
- *	2. kernel version is newer than 2.6.25 (because multiqueue is not
- *	   supporter) in kernels older than that)
- *	3. msi can be enabled. If it fails it means that MSI is not available.
- * When all the above are true, return non-zero so that multiple queues will be
- * allowed in the driver.
- */
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
-#   define compat_multiqueue_allowed(dev) pci_msi_enabled()
-#else
-#   if defined CONFIG_PCI_MSI && LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 25)
-static inline int
-compat_multiqueue_allowed(struct pci_dev *dev)
-{
-   int ret;
-
-   if (!pci_enable_msi(dev))
-      ret = 1;
-   else
-      ret = 0;
-
-   pci_disable_msi(dev);
-   return ret;
-}
-
-#   else
-#      define compat_multiqueue_allowed(dev) (0)
-#   endif
-#endif
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 37)
-#   define compat_vlan_get_protocol(skb) vlan_get_protocol(skb)
-#else
-#   define compat_vlan_get_protocol(skb) (skb->protocol)
-#endif
-
 #endif /* __COMPAT_NETDEVICE_H__ */
