@@ -53,10 +53,18 @@
 #include "vmware.h"
 #include "procMgr.h"
 #include "vm_version.h"
+#include "vm_app.h"
 #include "message.h"
 
-#include "vixPluginInt.h"
-#include "vmware/tools/utils.h"
+#if defined(VMTOOLS_USE_GLIB)
+#  include "vixPluginInt.h"
+#  include "vmtools.h"
+#else
+#  include "debug.h"
+#  include "eventManager.h"
+#  include "rpcin.h"
+#  include "rpcout.h"
+#endif
 
 #include "util.h"
 #include "strutil.h"
@@ -65,28 +73,38 @@
 #include "err.h"
 #include "hostinfo.h"
 #include "guest_os.h"
-#include "guest_msg_def.h"
 #include "conf.h"
 #include "vixCommands.h"
+#include "foundryToolsDaemon.h"
+#include "printer.h"
 #include "base64.h"
 #include "syncDriver.h"
-#include "hgfsServerManager.h"
+#include "hgfsServer.h"
 #include "hgfs.h"
 #include "system.h"
 #include "codeset.h"
-#include "vixToolsInt.h"
 
 #if defined(linux)
 #include "hgfsDevLinux.h"
 #endif
 
-/* Only Win32, Linux, Solaris and FreeBSD use impersonation functions. */
-#if !defined(__APPLE__)
+#ifndef __FreeBSD__
+#include "netutil.h"
+#endif
+
+/* Only Win32 and Linux use impersonation functions. */
+#if !defined(__FreeBSD__) && !defined(sun)
 #include "impersonate.h"
 #endif
 
+#include "vixTools.h"
 #include "vixOpenSource.h"
 
+#if !defined(VMTOOLS_USE_GLIB)
+static DblLnkLst_Links *globalEventQueue;   // event queue for main event loop
+#endif
+
+#define GUESTMSG_MAX_IN_SIZE (64 * 1024) /* vmx/main/guest_msg.c */
 #define MAX64_DECIMAL_DIGITS 20          /* 2^64 = 18,446,744,073,709,551,616 */
 
 #if defined(linux) || defined(_WIN32)
@@ -107,22 +125,22 @@ static Bool ToolsDaemonSyncDriverThawCallback(void *clientData);
 static char *ToolsDaemonTcloGetQuotedString(const char *args,
                                             const char **endOfArg);
   
-static VixError ToolsDaemonTcloGetEncodedQuotedString(const char *args,
-                                                      const char **endOfArg,
-                                                      char **result);
+static char * ToolsDaemonTcloGetEncodedQuotedString(const char *args,
+                                                    const char **endOfArg);
 
-gboolean ToolsDaemonTcloReceiveVixCommand(RpcInData *data);
+Bool ToolsDaemonTcloReceiveVixCommand(RpcInData *data);
 
-static HgfsServerMgrData gFoundryHgfsBkdrConn;
-gboolean ToolsDaemonHgfsImpersonated(RpcInData *data);
-
-#if defined(linux) || defined(_WIN32)
-gboolean ToolsDaemonTcloSyncDriverFreeze(RpcInData *data);
-
-gboolean ToolsDaemonTcloSyncDriverThaw(RpcInData *data);
+#if !defined(N_PLAT_NLM)
+Bool ToolsDaemonHgfsImpersonated(RpcInData *data);
 #endif
 
-gboolean ToolsDaemonTcloMountHGFS(RpcInData *data);
+#if defined(linux) || defined(_WIN32)
+Bool ToolsDaemonTcloSyncDriverFreeze(RpcInData *data);
+
+Bool ToolsDaemonTcloSyncDriverThaw(RpcInData *data);
+#endif
+
+Bool ToolsDaemonTcloMountHGFS(RpcInData *data);
 
 void ToolsDaemonTcloReportProgramCompleted(const char *requestName,
                                            VixError err,
@@ -140,6 +158,7 @@ void ToolsDaemonTcloReportProgramCompleted(const char *requestName,
 static Bool thisProcessRunsAsRoot = FALSE;
 
 
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -157,7 +176,7 @@ static Bool thisProcessRunsAsRoot = FALSE;
  *-----------------------------------------------------------------------------
  */
 
-gboolean
+Bool
 FoundryToolsDaemonRunProgram(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -172,26 +191,19 @@ FoundryToolsDaemonRunProgram(RpcInData *data) // IN
    Bool impersonatingVMWareUser = FALSE;
    void *userToken = NULL;
    ProcMgr_Pid pid;
+#if defined(VMTOOLS_USE_GLIB)
    GMainLoop *eventQueue = ((ToolsAppCtx *)data->appCtx)->mainLoop;
+#else
+   DblLnkLst_Links *eventQueue = data->clientData;
+#endif
 
    /*
     * Parse the arguments. Some of these are optional, so they
     * may be NULL.
     */
    requestName = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
-
-   err = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args,
-                                               &commandLine);
-   if (err != VIX_OK) {
-      goto abort;
-   }
-
-   err = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args,
-                                               &commandLineArgs);
-   if (err != VIX_OK) {
-      goto abort;
-   }
-
+   commandLine = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args);
+   commandLineArgs = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args);
    credentialTypeStr = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
    obfuscatedNamePassword = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
    directoryPath = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
@@ -278,7 +290,7 @@ abort:
  *-----------------------------------------------------------------------------
  */
 
-gboolean
+Bool
 FoundryToolsDaemonGetToolsProperties(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -290,7 +302,11 @@ FoundryToolsDaemonGetToolsProperties(RpcInData *data) // IN
    size_t base64BufferLength = 0;
    Bool success;
    char *returnBuffer = NULL;
+#if defined(VMTOOLS_USE_GLIB)
    GKeyFile *confDictRef;
+#else
+   GuestApp_Dict **confDictRef;
+#endif
    
    /*
     * Collect some values about the host.
@@ -345,6 +361,86 @@ abort:
 } // FoundryToolsDaemonGetToolsProperties
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * ToolsDaemonTcloCheckUserAccount --
+ *
+ *
+ * Return value:
+ *    TRUE on success
+ *    FALSE on failure
+ *
+ * Side effects:
+ *    None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+ToolsDaemonTcloCheckUserAccount(RpcInData *data) // IN
+{
+   VixError err = VIX_OK;
+   char *credentialTypeStr = NULL;
+   char *obfuscatedNamePassword = NULL;
+   static char resultBuffer[DEFAULT_RESULT_MSG_MAX_LENGTH];
+   Bool impersonatingVMWareUser = FALSE;
+   void *userToken = NULL;
+   Debug(">ToolsDaemonTcloCheckUserAccount\n");
+
+   /*
+    * Parse the argument
+    */
+   credentialTypeStr = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
+   obfuscatedNamePassword = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
+
+   /*
+    * Make sure we are passed the correct arguments.
+    */
+   if ((NULL == credentialTypeStr) || (NULL == obfuscatedNamePassword)) {
+      err = VIX_E_INVALID_ARG;
+      goto abort;
+   }
+
+   if (thisProcessRunsAsRoot) {
+      impersonatingVMWareUser = VixToolsImpersonateUserImpl(credentialTypeStr, 
+                                                            VIX_USER_CREDENTIAL_NONE,
+                                                            obfuscatedNamePassword, 
+                                                            &userToken);
+      if (!impersonatingVMWareUser) {
+         err = VIX_E_GUEST_USER_PERMISSIONS;
+         goto abort;
+      }
+   }
+
+abort:
+   if (impersonatingVMWareUser) {
+      VixToolsUnimpersonateUser(userToken);
+   }
+   VixToolsLogoutUser(userToken);
+
+   /*
+    * All VMXI tools commands return results that start with a VMXI error
+    * and a guest-OS-specific error.
+    */
+   Str_Sprintf(resultBuffer,
+               sizeof(resultBuffer),
+               "%"FMT64"d %d",
+               err,
+               Err_Errno());
+   RPCIN_SETRETVALS(data, resultBuffer, TRUE);
+
+   /*
+    * These were allocated by ToolsDaemonTcloGetQuotedString.
+    */
+   free(credentialTypeStr);
+   free(obfuscatedNamePassword);
+
+   return TRUE;
+} // ToolsDaemonTcloCheckUserAccount
+
+#if defined(VMTOOLS_USE_GLIB)
+
 /**
  * Initializes internal state of the Foundry daemon.
  *
@@ -362,43 +458,131 @@ FoundryToolsDaemon_Initialize(ToolsAppCtx *ctx)
     * changed them.
     */
    (void) VixTools_Initialize(thisProcessRunsAsRoot,
-#if defined(__FreeBSD__)
-                              ctx->envp,   // envp
-#else
-                              NULL,        // envp
-#endif
+                              NULL,   // envp
                               ToolsDaemonTcloReportProgramCompleted,
                               ctx);
 
-#if !defined(__APPLE__)
+#if defined(linux) || defined(_WIN32)
    if (thisProcessRunsAsRoot) {
       Impersonate_Init();
    }
 #endif
 
-   /* Register a straight through connection with the Hgfs server. */
-   HgfsServerManager_DataInit(&gFoundryHgfsBkdrConn,
-                              VIX_BACKDOORCOMMAND_SEND_HGFS_PACKET,
-                              NULL,    // rpc - no rpc registered
-                              NULL);   // rpc callback
-   HgfsServerManager_Register(&gFoundryHgfsBkdrConn);
-
 }
 
+#else
 
-/**
- * Uninitializes internal state of the Foundry daemon.
+/*
+ *-----------------------------------------------------------------------------
  *
- * @param[in]  ctx      Application context.
+ * FoundryToolsDaemon_RegisterRoutines --
+ *
+ *    Register the Foundry RPC callbacks
+ *
+ * Return value:
+ *    None
+ *
+ * Side effects:
+ *    None
+ *
+ *-----------------------------------------------------------------------------
  */
 
 void
-FoundryToolsDaemon_Uninitialize(ToolsAppCtx *ctx)
+FoundryToolsDaemon_RegisterRoutines(RpcIn *in,                        // IN
+                                    GuestApp_Dict **confDictRef,      // IN
+                                    DblLnkLst_Links *eventQueue,      // IN
+                                    const char * const *orginalEnvp,  // IN
+                                    Bool runAsRoot)                   // IN
 {
-   HgfsServerManager_Unregister(&gFoundryHgfsBkdrConn);
-   VixTools_Uninitialize();
-}
+   static Bool inited = FALSE;
+#if defined(linux) || defined(_WIN32)
+   static Bool sync_driver_inited = FALSE;
+#endif
 
+   ASSERT(in);
+   ASSERT(confDictRef);
+   ASSERT(*confDictRef);
+
+   thisProcessRunsAsRoot = runAsRoot;
+   globalEventQueue = eventQueue;
+
+   (void) VixTools_Initialize(thisProcessRunsAsRoot,
+                              orginalEnvp,
+                              ToolsDaemonTcloReportProgramCompleted,
+                              NULL);
+
+#if defined(linux) || defined(_WIN32)
+   /*
+    * Be careful, Impersonate_Init should only be ever called once per process.
+    *
+    * We can get back here if the tools re-inits due to an error state,
+    * such as hibernation.
+    */
+   if (!inited && thisProcessRunsAsRoot) {
+      Impersonate_Init();
+   }
+#endif
+
+   RpcIn_RegisterCallbackEx(in,
+                            VIX_BACKDOORCOMMAND_RUN_PROGRAM,
+                            FoundryToolsDaemonRunProgram,
+                            eventQueue);
+   RpcIn_RegisterCallbackEx(in,
+                            VIX_BACKDOORCOMMAND_GET_PROPERTIES,
+                            FoundryToolsDaemonGetToolsProperties,
+                            confDictRef);
+   RpcIn_RegisterCallbackEx(in,
+                            VIX_BACKDOORCOMMAND_CHECK_USER_ACCOUNT,
+                            ToolsDaemonTcloCheckUserAccount,
+                            NULL);
+#if !defined(N_PLAT_NLM)
+   RpcIn_RegisterCallbackEx(in,
+                            VIX_BACKDOORCOMMAND_SEND_HGFS_PACKET,
+                            ToolsDaemonHgfsImpersonated,
+                            NULL);
+#endif
+   RpcIn_RegisterCallbackEx(in,
+                            VIX_BACKDOORCOMMAND_COMMAND,
+                            ToolsDaemonTcloReceiveVixCommand,
+                            confDictRef);
+   RpcIn_RegisterCallbackEx(in,
+                            VIX_BACKDOORCOMMAND_MOUNT_VOLUME_LIST,
+                            ToolsDaemonTcloMountHGFS,
+                            NULL);
+
+#if defined(linux) || defined(_WIN32)
+
+   /*
+    * Only init once, but always register the RpcIn.
+    */
+   if (!sync_driver_inited) {
+      sync_driver_inited = SyncDriver_Init();
+   }
+   if (sync_driver_inited) {
+      /*
+       * These only get registered if SyncDriver_Init succeeds. We do
+       * support running the sync/thaw scripts even on guests where
+       * the Sync driver is not supported (Linux, Windows older than
+       * Win2k) but the running of the scripts is implemented using
+       * VIX_BACKDOORCOMMAND_RUN_PROGRAM.
+       */
+      RpcIn_RegisterCallbackEx(in,
+                               VIX_BACKDOORCOMMAND_SYNCDRIVER_FREEZE,
+                               ToolsDaemonTcloSyncDriverFreeze,
+                               eventQueue);
+      RpcIn_RegisterCallbackEx(in,
+                               VIX_BACKDOORCOMMAND_SYNCDRIVER_THAW,
+                               ToolsDaemonTcloSyncDriverThaw,
+                               NULL);
+   } else {
+      Debug("FoundryToolsDaemon: Failed to init SyncDriver, skipping command handlers.\n");
+   }
+#endif
+   inited = TRUE;
+} // FoundryToolsDaemon_RegisterRoutines
+
+#endif
 
 /*
  *-----------------------------------------------------------------------------
@@ -483,29 +667,124 @@ ToolsDaemonTcloGetQuotedString(const char *args,      // IN
  *-----------------------------------------------------------------------------
  */
 
-static VixError
+static char *
 ToolsDaemonTcloGetEncodedQuotedString(const char *args,      // IN
-                                      const char **endOfArg, // OUT
-                                      char **result)         // OUT
+                                      const char **endOfArg) // OUT
 {
-   VixError err = VIX_OK;
    char *rawResultStr = NULL;
    char *resultStr = NULL;
 
    rawResultStr = ToolsDaemonTcloGetQuotedString(args, endOfArg);
    if (NULL == rawResultStr) {
+      return(NULL);
+   }
+
+   resultStr = VixMsg_DecodeString(rawResultStr);
+   free(rawResultStr);
+
+   return resultStr;
+}
+
+
+#if !defined(VMTOOLS_USE_GLIB)
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * ToolsDaemonTcloOpenUrl --
+ *
+ *    Open a URL on the guest.
+ *
+ * Return value:
+ *    TRUE on success
+ *    FALSE on failure
+ *
+ * Side effects:
+ *    None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+ToolsDaemonTcloOpenUrl(RpcInData *data) // IN
+{
+   static char resultBuffer[DEFAULT_RESULT_MSG_MAX_LENGTH];
+   VixError err = VIX_OK;
+   char *url = NULL;
+   char *windowState = NULL;
+   char *credentialTypeStr = NULL;
+   char *obfuscatedNamePassword = NULL;
+   uint32 sysError = 0;
+   Bool impersonatingVMWareUser = FALSE;
+   void *userToken = NULL;
+   Debug(">ToolsDaemonTcloOpenUrl\n");
+
+   /*
+    * Parse the arguments
+    */
+   url = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args);
+   windowState = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
+   // These parameters at the end are optional, so they may be NULL.
+   credentialTypeStr = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
+   obfuscatedNamePassword = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
+
+   /*
+    * Validate the arguments.
+    */
+   if ((NULL == url) || (NULL == windowState)) {
+      err = VIX_E_INVALID_ARG;
+      Debug("Failed to get string args\n");
       goto abort;
    }
 
-   err = VixMsg_DecodeString(rawResultStr, &resultStr);
+   if ((NULL != credentialTypeStr) 
+         && (*credentialTypeStr)
+         && (thisProcessRunsAsRoot)) {
+      impersonatingVMWareUser = VixToolsImpersonateUserImpl(credentialTypeStr, 
+                                                            VIX_USER_CREDENTIAL_NONE,
+                                                            obfuscatedNamePassword, 
+                                                            &userToken);
+      if (!impersonatingVMWareUser) {
+         err = VIX_E_GUEST_USER_PERMISSIONS;
+         goto abort;
+      }
+   }
+
+   Debug("Opening URL: \"%s\"\n", url);
+
+   /* Actually open the URL. */
+   if (!GuestApp_OpenUrl(url, strcmp(windowState, "maximize") == 0)) {
+      err = VIX_E_FAIL;
+      Debug("Failed to open the url \"%s\"\n", url);
+      goto abort;
+   }
 
 abort:
-   free(rawResultStr);
-   *result = resultStr;
+   if (impersonatingVMWareUser) {
+      VixToolsUnimpersonateUser(userToken);
+   }
+   VixToolsLogoutUser(userToken);
 
-   return err;
+   /*
+    * All Foundry tools commands return results that start with a
+    * foundry error and a guest-OS-specific error.
+    */
+   Str_Sprintf(resultBuffer, sizeof resultBuffer, "%"FMT64"d %d", err, sysError);
+   RPCIN_SETRETVALS(data, resultBuffer, TRUE);
+
+   /*
+    * These were allocated by ToolsDaemonTcloGetQuotedString.
+    */
+   free(url);
+   free(windowState);
+   free(credentialTypeStr);
+   free(obfuscatedNamePassword);
+
+   Debug("<ToolsDaemonTcloOpenUrl\n");
+   return TRUE;
 }
 
+#endif
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -525,7 +804,7 @@ abort:
  */
 
 #if defined(linux) || defined(_WIN32)
-gboolean
+Bool
 ToolsDaemonTcloSyncDriverFreeze(RpcInData *data)
 {
    static char resultBuffer[DEFAULT_RESULT_MSG_MAX_LENGTH];
@@ -534,8 +813,12 @@ ToolsDaemonTcloSyncDriverFreeze(RpcInData *data)
    char *timeout = NULL;
    int timeoutVal;
    DECLARE_SYNCDRIVER_ERROR(sysError);
+#if defined(VMTOOLS_USE_GLIB)
    ToolsAppCtx *ctx = data->appCtx;
    GSource *timer;
+#else
+   Event *cbEvent;
+#endif
    
    Debug(">ToolsDaemonTcloSyncDriverFreeze\n");
 
@@ -586,9 +869,26 @@ ToolsDaemonTcloSyncDriverFreeze(RpcInData *data)
    /* Start the timer callback to automatically thaw. */
    if (0 != timeoutVal) {
       Debug("ToolsDaemonTcloSyncDriverFreeze: Starting timer callback %d\n", timeoutVal);
+#if defined(VMTOOLS_USE_GLIB)
       timer = g_timeout_source_new(timeoutVal * 10);
       VMTOOLSAPP_ATTACH_SOURCE(ctx, timer, ToolsDaemonSyncDriverThawCallback, NULL, NULL);
       g_source_unref(timer);
+#else
+      cbEvent = EventManager_Add(data->clientData,
+                                 timeoutVal,
+                                 ToolsDaemonSyncDriverThawCallback,
+                                 NULL);
+      if (!cbEvent) {
+         Debug("ToolsDaemonTcloSyncDriverFreeze: Failed to start callback, aborting\n");
+         if (!SyncDriver_Thaw(gSyncDriverHandle)) {
+            Debug("ToolsDaemonTcloSyncDriverFreeze: Unable to abort freeze. Oh well.\n");
+         }
+         SyncDriver_CloseHandle(&gSyncDriverHandle);
+         err = VIX_E_FAIL;
+         sysError = SYNCDRIVERERROR;
+         goto abort;
+      }
+#endif
    }
 
 abort:
@@ -671,7 +971,7 @@ exit:
  */
 
 #if defined(linux) || defined(_WIN32)
-gboolean
+Bool
 ToolsDaemonTcloSyncDriverThaw(RpcInData *data) // IN
 {
    static char resultBuffer[DEFAULT_RESULT_MSG_MAX_LENGTH];
@@ -708,6 +1008,125 @@ ToolsDaemonTcloSyncDriverThaw(RpcInData *data) // IN
 }
 #endif
 
+#if !defined(VMTOOLS_USE_GLIB)
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * FoundryToolsDaemon_RegisterOpenUrlCapability --
+ *
+ *      Register the OpenUrl capability. Sometimes this needs to
+ *      be done separately from the TCLO callback registration, so we
+ *      provide it separately here.
+ *
+ * Results:
+ *      TRUE on success
+ *      FALSE on failure
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+FoundryToolsDaemon_RegisterOpenUrlCapability(void)
+{
+   if (!RpcOut_sendOne(NULL, NULL, "tools.capability.open_url 1")) {
+      Debug("Unable to register open url capability\n");
+      Debug("<FoundryToolsDaemon_RegisterOpenUrlCapability");
+      return FALSE;
+   }
+
+   return TRUE;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * FoundryToolsDaemon_RegisterOpenUrl --
+ *
+ *      Register the OpenUrl capability and TCLO handler.
+ *
+ * Results:
+ *      TRUE on success
+ *      FALSE on failure
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+FoundryToolsDaemon_RegisterOpenUrl(RpcIn *in) // IN
+{
+   /* Register the TCLO handler. */
+   RpcIn_RegisterCallbackEx(in,
+                            VIX_BACKDOORCOMMAND_OPEN_URL,
+                            ToolsDaemonTcloOpenUrl,
+                            NULL);
+   /*
+    * Inform the VMX that we support opening urls in the guest; the UI
+    * and VMX need to know about this capability in advance (rather than
+    * simply trying and failing). I've put this here on the theory that
+    * it's better to have it close to the command that handles the
+    * actual request than it is to have it near the other capabilities
+    * registration code, which is in toolsDaemon.c.
+    *
+    * Eventually, Foundry might want to have a unified way of
+    * registering support for only a subset of the foundry commands in
+    * the guest.
+    */
+   if (!FoundryToolsDaemon_RegisterOpenUrlCapability()) {
+      // Unregister Callback to avoid double registration down the line
+      RpcIn_UnregisterCallback(in, VIX_BACKDOORCOMMAND_OPEN_URL);
+      return FALSE;
+   }
+
+   return TRUE;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * FoundryToolsDaemon_UnregisterOpenUrl --
+ *
+ *      Unregister the "OpenUrl" capability.
+ *
+ * Results:
+ *      TRUE on success
+ *      FALSE on failure
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+FoundryToolsDaemon_UnregisterOpenUrl(void) 
+{
+   /*
+    * RpcIn doesn't have an unregister facility, so all we need to do
+    * here is unregister the capability.
+    */
+
+   /*
+    * Report no longer supporting open url;
+    */
+   if (!RpcOut_sendOne(NULL, NULL, "tools.capability.open_url 0")) {
+      Debug("<FoundryToolsDaemon_UnregisterOpenUrl\n");
+      Debug("Unable to unregister OpenUrl capability\n");
+      return FALSE;
+   }
+
+   return TRUE;
+}
+
+#endif
 
 /*
  *-----------------------------------------------------------------------------
@@ -724,7 +1143,7 @@ ToolsDaemonTcloSyncDriverThaw(RpcInData *data) // IN
  *-----------------------------------------------------------------------------
  */
 
-gboolean
+Bool
 ToolsDaemonTcloMountHGFS(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -823,12 +1242,11 @@ ToolsDaemonTcloMountHGFS(RpcInData *data) // IN
  *-----------------------------------------------------------------------------
  */
 
-gboolean
+Bool
 ToolsDaemonHgfsImpersonated(RpcInData *data) // IN
 {
    VixError err;
    size_t hgfsPacketSize = 0;
-   size_t hgfsReplySize = 0;
    const char *origArgs = data->args;
    Bool impersonatingVMWareUser = FALSE;
    char *credentialTypeStr = NULL;
@@ -906,6 +1324,7 @@ ToolsDaemonHgfsImpersonated(RpcInData *data) // IN
                                                             &userToken);
       if (!impersonatingVMWareUser) {
          err = VIX_E_GUEST_USER_PERMISSIONS;
+         hgfsPacketSize = 0;
          goto abort;
       }
    }
@@ -914,12 +1333,10 @@ ToolsDaemonHgfsImpersonated(RpcInData *data) // IN
     * Impersonation was okay, so let's give our packet to
     * the HGFS server and forward the reply packet back.
     */
-   hgfsReplySize = sizeof resultPacket - (hgfsReplyPacket - resultPacket);
-   HgfsServerManager_ProcessPacket(&gFoundryHgfsBkdrConn, // hgfs server connection
-                                   data->args,            // packet in buf
-                                   hgfsPacketSize,        // packet in size
-                                   hgfsReplyPacket,       // packet out buf
-                                   &hgfsReplySize);       // reply buf/data size
+   HgfsServer_ProcessPacket(data->args,        // packet in buf
+                            hgfsReplyPacket,   // packet out buf
+                            &hgfsPacketSize,   // in/out size
+                            0);                // receive process flags
 
 abort:
    if (impersonatingVMWareUser) {
@@ -936,7 +1353,7 @@ abort:
    data->result = resultPacket;
    data->resultLen = STRLEN_OF_MAX_64_BIT_NUMBER_AS_STRING
                         + OTHER_TEXT_SIZE
-                        + hgfsReplySize;
+                        + hgfsPacketSize;
    
    /*
     * Render the foundry error codes into the buffer.
@@ -1016,6 +1433,8 @@ ToolsDaemonTcloReportProgramCompleted(const char *requestName,    // IN
                                       void *clientData)           // IN
 {
    Bool sentResult;
+
+#if defined(VMTOOLS_USE_GLIB)
    ToolsAppCtx *ctx = clientData;
    gchar *msg = g_strdup_printf("%s %s %"FMT64"d %d %d %"FMT64"d",
                                 VIX_BACKDOORCOMMAND_RUN_PROGRAM_DONE,
@@ -1024,9 +1443,19 @@ ToolsDaemonTcloReportProgramCompleted(const char *requestName,    // IN
                                 Err_Errno(),
                                 exitCode,
                                 (int64) pid);
-
    sentResult = RpcChannel_Send(ctx->rpc, msg, strlen(msg) + 1, NULL, NULL);
    g_free(msg);
+#else
+   sentResult = RpcOut_sendOne(NULL,
+                               NULL,
+                               "%s %s %"FMT64"d %d %d %"FMT64"d",
+                               VIX_BACKDOORCOMMAND_RUN_PROGRAM_DONE,
+                               requestName,
+                               err,
+                               Err_Errno(),
+                               exitCode,
+                               (int64) pid);
+#endif
 
    if (!sentResult) {
       Warning("Unable to send results from polling the result program.\n\n");
@@ -1050,7 +1479,7 @@ ToolsDaemonTcloReportProgramCompleted(const char *requestName,    // IN
  *-----------------------------------------------------------------------------
  */
 
-gboolean
+Bool
 ToolsDaemonTcloReceiveVixCommand(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -1064,8 +1493,7 @@ ToolsDaemonTcloReceiveVixCommand(RpcInData *data) // IN
    char *destPtr = NULL;
    int vixPrefixDataSize = (MAX64_DECIMAL_DIGITS * 2)
                              + (sizeof(' ') * 2)
-                             + sizeof('\0')
-                             + sizeof(' ') * 10;   // for RPC header
+                             + sizeof('\0');
 
    /*
     * Our temporary buffer will be the same size as what the
@@ -1073,9 +1501,14 @@ ToolsDaemonTcloReceiveVixCommand(RpcInData *data) // IN
     */
    static char tcloBuffer[GUESTMSG_MAX_IN_SIZE];
 
+#if defined(VMTOOLS_USE_GLIB)
    ToolsAppCtx *ctx = data->appCtx;
    GMainLoop *eventQueue = ctx->mainLoop;
    GKeyFile *confDictRef = ctx->config;
+#else
+   DblLnkLst_Links *eventQueue = globalEventQueue;
+   GuestApp_Dict **confDictRef = data->clientData;
+#endif
 
    requestName = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
 
@@ -1118,6 +1551,18 @@ abort:
       err = VIX_E_OUT_OF_MEMORY;
    }
 
+   /*
+    * This should never happen since tcloBuffer is pretty huge.
+    * But, there is no harm in being paranoid and I don't want to
+    * find out the buffer is too small when we are halfway through
+    * formatting the response.
+    */
+   if ((32 + resultValueLength + 32) >= sizeof(tcloBuffer)) {
+      err = VIX_E_OUT_OF_MEMORY;
+      resultValueLength = 0;
+   }
+
+
    /*
     * All Foundry tools commands return results that start with a foundry error
     * and a guest-OS-specific error.
